package ru.dbotthepony.mc.otm.item

import it.unimi.dsi.fastutil.ints.Int2ObjectArrayMap
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.Container
import net.minecraft.world.InteractionHand
import net.minecraft.world.InteractionResult
import net.minecraft.world.item.BlockItem
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.context.BlockPlaceContext
import net.minecraft.world.item.context.UseOnContext
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.entity.BarrelBlockEntity
import net.minecraft.world.level.block.entity.ChestBlockEntity
import net.minecraft.world.level.storage.loot.LootParams
import net.minecraft.world.level.storage.loot.parameters.LootContextParams
import ru.dbotthepony.mc.otm.block.decorative.CargoCrateBlock
import ru.dbotthepony.mc.otm.block.entity.decorative.CargoCrateBlockEntity
import ru.dbotthepony.mc.otm.core.math.Vector

class ChestUpgraderItem : Item(Properties().stacksTo(1)) {
	override fun onItemUseFirst(stack: ItemStack, context: UseOnContext): InteractionResult {
		val player = context.player ?: return super.onItemUseFirst(stack, context)

		val otherHand = if (context.hand == InteractionHand.MAIN_HAND) InteractionHand.OFF_HAND else InteractionHand.MAIN_HAND
		val crateStack = player.getItemInHand(otherHand) ?: return super.onItemUseFirst(stack, context)

		val block : Block
		if (!crateStack.isEmpty && crateStack.item is BlockItem && (crateStack.item as BlockItem).block is CargoCrateBlock) {
			block = (crateStack.item as BlockItem).block
		} else {
			return super.onItemUseFirst(stack, context)
		}

		val pos = context.clickedPos
		val hitBlockEntity = context.level.getBlockEntity(pos)

		if (hitBlockEntity is ChestBlockEntity || hitBlockEntity is BarrelBlockEntity) {
			val container = hitBlockEntity as Container

			if (container.containerSize >= 54) return super.onItemUseFirst(stack, context)

			if (context.level is ServerLevel) {
				val newState = block.getStateForPlacement(BlockPlaceContext(context)) ?: return InteractionResult.FAIL

				val contents = Int2ObjectArrayMap<ItemStack>(container.containerSize)
				for (i in 0 until container.containerSize) {
					contents.put(i, container.getItem(i))
				}
				container.clearContent()

				val level = context.level as ServerLevel

				val blockState = level.getBlockState(pos)
				val lootparams = LootParams.Builder(level)
					.withParameter(LootContextParams.ORIGIN, Vector.atCenterOf(context.clickedPos))
					.withParameter(LootContextParams.TOOL, stack)
					.withOptionalParameter(LootContextParams.BLOCK_ENTITY, hitBlockEntity)
					.withOptionalParameter(LootContextParams.BLOCK_STATE, blockState)
					.withOptionalParameter(LootContextParams.THIS_ENTITY, context.player)

				blockState.spawnAfterBreak(level, pos, stack, true)
				blockState.getDrops(lootparams).forEach { Block.popResource(level, pos, it) }

				if (!player.isCreative)
					player.getItemInHand(otherHand).shrink(1)

				level.setBlockAndUpdate(pos, newState)

				val newBlockEntity = level.getBlockEntity(pos)
				if (newBlockEntity is CargoCrateBlockEntity) {
					for (i in 0 until contents.size) {
						newBlockEntity.container[i] = contents.get(i)
					}
				} else {
					for (i in 0 until contents.size) {
						Block.popResource(level, pos, contents.get(i))
					}
				}
			}

			return InteractionResult.SUCCESS
		}

		return super.onItemUseFirst(stack, context)
	}
}
